@@ -38,9 +38,9 @@
 #include <stdlib.h>
 
 gboolean
-eel_uri_is_favorites (const gchar *uri)
+eel_uri_is_starred (const gchar *uri)
 {
-    return g_str_has_prefix (uri, "favorites:");
+    return g_str_has_prefix (uri, "starred:");
 }
 
 gboolean
@@ -67,6 +67,38 @@ eel_uri_is_other_locations (const char *uri)
     return g_str_has_prefix (uri, "other-locations:");
 }
 
+gboolean
+eel_uri_is_in_xdg_dirs (const gchar *uri)
+{
+    GUserDirectory dir;
+    g_autoptr (GFile) location = NULL;
+    gboolean has_prefix = FALSE;
+
+    location = g_file_new_for_uri (uri);
+    for (dir = 0; dir < G_USER_N_DIRECTORIES; dir++)
+    {
+        g_autoptr (GFile) xdg_dir_location = NULL;
+        const gchar *path;
+
+        path = g_get_user_special_dir (dir);
+        if (path == NULL)
+        {
+            continue;
+        }
+
+        xdg_dir_location = g_file_new_for_path (path);
+        has_prefix = g_file_has_prefix (location, xdg_dir_location) ||
+                     g_file_equal (location, xdg_dir_location);
+
+        if (has_prefix)
+        {
+            break;
+        }
+    }
+
+    return has_prefix;
+}
+
 char *
 eel_filename_get_extension_offset (const char *filename)
 {
