@@ -27,7 +27,6 @@
 #include "nautilus-directory-private.h"
 #include "nautilus-file.h"
 #include "nautilus-ui-utilities.h"
-#include "nautilus-tag-manager.h"
 #define DEBUG_FLAG NAUTILUS_DEBUG_SEARCH
 #include "nautilus-debug.h"
 
@@ -131,8 +130,9 @@ model_directory_ready_cb (NautilusDirectory *directory,
                           gpointer           user_data)
 {
     NautilusSearchEngineModel *model = user_data;
+    g_autoptr (GPtrArray) mime_types = NULL;
     gchar *uri, *display_name;
-    GList *files, *hits, *mime_types, *l, *m;
+    GList *files, *hits, *l;
     NautilusFile *file;
     gdouble match;
     gboolean found;
@@ -140,7 +140,6 @@ model_directory_ready_cb (NautilusDirectory *directory,
     GDateTime *initial_date;
     GDateTime *end_date;
     GPtrArray *date_range;
-    NautilusTagManager *tag_manager;
 
     files = nautilus_directory_get_file_list (directory);
     mime_types = nautilus_query_get_mime_types (model->query);
@@ -154,13 +153,13 @@ model_directory_ready_cb (NautilusDirectory *directory,
         match = nautilus_query_matches_string (model->query, display_name);
         found = (match > -1);
 
-        if (found && mime_types)
+        if (found && mime_types->len > 0)
         {
             found = FALSE;
 
-            for (m = mime_types; m != NULL; m = m->next)
+            for (gint i = 0; i < mime_types->len; i++)
             {
-                if (nautilus_file_is_mime_type (file, m->data))
+                if (nautilus_file_is_mime_type (file, g_ptr_array_index (mime_types, i)))
                 {
                     found = TRUE;
                     break;
@@ -193,20 +192,6 @@ model_directory_ready_cb (NautilusDirectory *directory,
             g_ptr_array_unref (date_range);
         }
 
-        if (nautilus_query_get_search_starred (model->query))
-        {
-            tag_manager = nautilus_tag_manager_get ();
-
-            uri = nautilus_file_get_uri (file);
-
-            if (!nautilus_tag_manager_file_is_starred (tag_manager, uri))
-            {
-                found = FALSE;
-            }
-
-            g_free (uri);
-        }
-
         if (found)
         {
             uri = nautilus_file_get_uri (file);
@@ -220,7 +205,6 @@ model_directory_ready_cb (NautilusDirectory *directory,
         g_free (display_name);
     }
 
-    g_list_free_full (mime_types, g_free);
     nautilus_file_list_free (files);
     model->hits = hits;
 
